import pytest
from pytorch_nn_tools.train.progress import ProgressTracker


def test_progress_tracker__total_count():
    tracker = ProgressTracker()
    for i in tracker.track((1, 2, 3)):
        assert tracker.cnt_total_iter == i

    assert tracker.cnt_total_iter == 3, "counts must be preserved outside of the loop"
    for i in tracker.track((1, 2, 3, 4)):
        assert tracker.cnt_total_iter == i + 3, "counting must continue for the next sequence"

    assert tracker.cnt_total_iter == 7, "counts must be preserved outside of the loop"


def test_progress_tracker__count_in_epoch():
    tracker = ProgressTracker()
    for i in tracker.track((1, 2, 3)):
        assert tracker.cnt_iter_in_epoch == i
    assert tracker.cnt_iter_in_epoch == 3, "counts must be preserved outside of the loop"

    for i in tracker.track((1, 2, 3, 4)):
        assert tracker.cnt_iter_in_epoch == i
    assert tracker.cnt_iter_in_epoch == 4


def test_progress_tracker_preserves_len():
    tracker = ProgressTracker()
    with pytest.raises(ValueError):
        len(tracker)
    it = iter(tracker.track((1, 2, 3)))
    assert len(it) == 3
    next(it)
    assert len(it) == 3


def test_progress_tracker__doesnt_give_len_if_iterable_doesnt_have_it():
    tracker = ProgressTracker()
    with pytest.raises(ValueError):
        len(tracker)
    it = iter(tracker.track((i for i in range(3))))
    with pytest.raises(TypeError):
        len(it)




from typing import List
import torch


def topk_accuracy(output, target, topk=(1,)) -> List:
    """
    Computes the accuracy over the k top predictions for a
    set of k.
    By default computes top-1 accuracy only.
    Accuracy is returned as a fraction from interval [0, 1].
    """
    with torch.no_grad():
        maxk = max(topk)
        batch_size = target.size(0)

        _, pred = output.topk(maxk, 1, True, True)
        pred = pred.t()
        correct = pred.eq(target.view(1, -1).expand_as(pred))

        res = [
            correct[:k].reshape(-1).float().sum(0, keepdim=True).div_(batch_size)
            for k in topk
        ]
        return res



from pytorch_nn_tools.metrics import accuracy
from torch import tensor


def test_topk_accuracy():
    output = tensor([[0.1, 0.2, 0.5], [0.8, 0.1, 0.2]])
    target = tensor([1, 0])
    results = accuracy.topk_accuracy(output, target, (1, 2, 3))
    assert results == [0.5, 1., 1.]



class ProgressTracker:
    def __init__(self):
        """
        Keeps track of global number of iterations and number of iterations withing an epoch.
        Epoch is defined by the current sequence.
        Use as a wrapper around an iterator or a sequence.
        >>> tracker = ProgressTracker()
        >>> for i in tracker.track((1,2,3)):
        ...     assert tracker.cnt_total_iter == i
        ...     assert tracker.cnt_iter_in_epoch == i
        >>> # counts are preserved outside of the loop
        >>> assert tracker.cnt_total_iter == 3
        >>> assert tracker.cnt_iter_in_epoch == 3
        >>> # counting continues for the next sequence.
        >>> for i in tracker.track((1,2,3,4)):
        ...     assert tracker.cnt_total_iter == i + 3
        ...     assert tracker.cnt_iter_in_epoch == i
        >>> assert tracker.cnt_total_iter == 7
        >>> assert tracker.cnt_iter_in_epoch == 4
        """
        self.cnt_total_iter = 0
        self.cnt_iter_in_epoch = 0
        self._current_iterable = None

    def track(self, dl):
        self._current_iterable = dl
        return self

    def __next__(self):
        if self._current_iterable is None:
            raise ValueError("Iteration was not initialized. Use track(iterable) to start iteration.")
        result = next(self._current_iterator)
        self.cnt_total_iter += 1
        self.cnt_iter_in_epoch += 1
        return result

    def __iter__(self):
        self.cnt_iter_in_epoch = 0
        self._current_iterator = iter(self._current_iterable)
        return self

    def __len__(self):
        if self._current_iterable is None:
            raise ValueError("Iteration was not initialized. Use track(iterable) to start iteration.")
        return len(self._current_iterable)






#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

requirements = [
    'Click>=7.0',
    'torch>=1.0',
    'torchvision',
    'pillow',
    'dataclasses; python_version < "3.7"',
    'tensorboard',
]

setup_requirements = ['pytest-runner', ]

test_requirements = ['pytest>=3', ]

setup(
    author="SergeM",
    author_email='serge-m@users.noreply.github.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Tools for NN creation with Pytorch",
    entry_points={
        'console_scripts': [
            #'pytorch_nn_tools=pytorch_nn_tools.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords='pytorch_nn_tools',
    name='pytorch_nn_tools',
    packages=find_packages(include=['pytorch_nn_tools', 'pytorch_nn_tools.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/serge-m/pytorch_nn_tools',
    version='0.3.7',
    zip_safe=False,
)
